@@ -1,10 +1,13 @@
-# Adapted from https://github.com/nmi-lab/torchneuromorphic by Emre Neftci and Clemens Schaefer
+# Adapted from https://github.com/nmi-lab/torchneuromorphic
+# by Emre Neftci and Clemens Schaefer
 
 import numpy as np
 import pandas as pd
 import torch
 import bisect
 
+from torchvision.transforms import Compose, Normalize, Lambda
+
 
 def find_first(a, tgt):
     return bisect.bisect_left(a, tgt)
@@ -36,7 +39,8 @@ class toDtype(object):
 class Downsample(object):
     """Resize the address event Tensor to the given size.
     Args:
-        factor: : Desired resize factor. Applied to all dimensions including time
+        factor: : Desired resize factor. Applied to all dimensions
+        including time
     """
 
     def __init__(self, factor):
@@ -47,7 +51,10 @@ class Downsample(object):
         return tmad // self.factor
 
     def __repr__(self):
-        return self.__class__.__name__ + "(dt = {0}, dp = {1}, dx = {2}, dy = {3})"
+        return (
+            self.__class__.__name__
+            + "(dt = {0}, dp = {1}, dx = {2}, dy = {3})"
+        )
 
 
 # class Crop(object):
@@ -70,7 +77,8 @@ class Downsample(object):
 
 # class ShuffleMask(object):
 #     '''
-#     Shuffles events within t_min and t_max with a Random spike train having the same number of events
+#     Shuffles events within t_min and t_max with a Random spike train
+#     having the same number of events
 #     '''
 #     def __init__(self, t_min, t_max, size=[2,32,32]):
 #         from decolle.snn_utils import spiketrains
@@ -83,7 +91,8 @@ class Downsample(object):
 #         idx = np.where((tmad[:,0]>=self.t_min) * (tmad[:,0]<self.t_max))
 #         for i in range(1,tmad.shape[1]):
 #             #tmad[idx,i] = shuffle_along_axis(tmad[idx,i][0],0)
-#             tmad[idx,i] = np.random.randint(low=0,high=self.size[i-1],size=len(idx[0]))
+#             tmad[idx,i] = np.random.randint(low=0,high=self.size[i-1],
+#             size=len(idx[0]))
 
 #         return tmad
 
@@ -174,7 +183,8 @@ class ToChannelHeightWidth(object):
 
         else:
             raise TypeError(
-                "Wrong number of dimensions. Found {0}, expected 1 or 3".format(n - 1)
+                "Wrong number of dimensions. Found {0}, "
+                "expected 1 or 3".format(n - 1)
             )
 
     def __repr__(self):
@@ -183,7 +193,8 @@ class ToChannelHeightWidth(object):
 
 class ToCountFrame(object):
     """Convert Address Events to Binary tensor.
-    Converts a numpy.ndarray (T x H x W x C) to a torch.FloatTensor of shape (T x C x H x W) in the range [0., 1., ...]
+    Converts a numpy.ndarray (T x H x W x C) to a torch.FloatTensor of shape
+    (T x C x H x W) in the range [0., 1., ...]
     """
 
     def __init__(self, T=500, size=[2, 32, 32]):
@@ -193,8 +204,8 @@ class ToCountFrame(object):
 
     def __call__(self, tmad):
         times = tmad[:, 0]
-        t_start = times[0]
-        t_end = times[-1]
+        # t_start = times[0]
+        # t_end = times[-1]
         addrs = tmad[:, 1:]
 
         ts = range(0, self.T)
@@ -216,7 +227,8 @@ class ToCountFrame(object):
 
 class ToEventSum(object):
     """Convert Address Events to Image By Summing.
-    Converts a numpy.ndarray (T x H x W x C) to a torch.FloatTensor of shape (C x H x W) in the range [0., 1., ...]
+    Converts a numpy.ndarray (T x H x W x C) to a torch.FloatTensor of shape
+    (C x H x W) in the range [0., 1., ...]
     """
 
     def __init__(self, T=500, size=[2, 32, 32]):
@@ -226,8 +238,8 @@ class ToEventSum(object):
 
     def __call__(self, tmad):
         times = tmad[:, 0]
-        t_start = times[0]
-        t_end = times[-1]
+        # t_start = times[0]
+        # t_end = times[-1]
         addrs = tmad[:, 1:]
 
         ts = range(0, self.T)
@@ -275,7 +287,9 @@ class FilterEvents(object):
 
 
 class ExpFilterEvents(FilterEvents):
-    def __init__(self, length, tau=200, channels=2, tpad=None, device="cpu", **kwargs):
+    def __init__(
+        self, length, tau=200, channels=2, tpad=None, device="cpu", **kwargs
+    ):
         t = torch.arange(0.0, length, 1.0)
         kernel = torch.ones(channels, 1, len(t), 1, 1)
         exp_kernel = torch.exp(-t / tau)
@@ -320,6 +334,7 @@ class hflip(object):
 
     Returns:
         PIL Image or Tensor:  Horizontally flipped image.
+
     """
 
     def __call__(self, target):
@@ -349,7 +364,8 @@ class dvs_permute(object):
 
 class Repeat(object):
     """
-    Replicate np.array (C) as (n_repeat X C). This is useful to transform sample labels into sequences
+    Replicate np.array (C) as (n_repeat X C). This is useful to transform
+    sample labels into sequences
     """
 
     def __init__(self, n_repeat):
@@ -376,6 +392,7 @@ class ToTensor(object):
             frame (numpy.ndarray): numpy array of frames
         Returns:
             Tensor: Converted data.
+
         """
         return torch.FloatTensor(frame).to(self.device)
 
