@@ -1,19 +1,16 @@
 import os
 import torch
 import torch.utils.data as data
-from torchvision.datasets.utils import (
-    verify_str_arg,
-    check_integrity,
-    gen_bar_updater,
-)
 from ._transforms import Compose
 from tqdm import tqdm
 import tarfile
 import zipfile
 import gzip
+import hashlib
 
 
-# Adapted from https://github.com/nmi-lab/torchneuromorphic by Emre Neftci and Clemens Schaefer
+# Adapted from https://github.com/nmi-lab/torchneuromorphic
+# by Emre Neftci and Clemens Schaefer
 
 DEFAULT_ROOT = "data/"
 
@@ -28,11 +25,13 @@ def download_url(url, root, filename=None, md5=None, total_size=None):
     :param root: Directory to place downloaded file in
     :type root: string
 
-    :param filename: Name to save the file under. If None, use the basename of the URL
+    :param filename: Name to save the file under. If None, use the
+        basename of the URL
     :type filename: string, optional
 
     :param md5: MD5 checksum of the download. If None, do not check
     :type md5: string, optional
+
     """
 
     from six.moves import urllib
@@ -56,8 +55,10 @@ def download_url(url, root, filename=None, md5=None, total_size=None):
                 headers = {"user-agent": "Wget/1.16 (linux-gnu)"}
                 r = requests.get(url, stream=True, headers=headers)
                 # # new
-                total_size_in_bytes = int(r.headers.get("content-length", 0))  # new
-                block_size = 1024  # 1 Kibibyte - new
+                total_size_in_bytes = int(
+                    r.headers.get("content-length", 0)
+                )  # new
+                # block_size = 1024  # 1 Kibibyte - new
                 progress_bar = tqdm(
                     total=total_size_in_bytes, unit="iB", unit_scale=True
                 )  # new
@@ -68,16 +69,22 @@ def download_url(url, root, filename=None, md5=None, total_size=None):
                             progress_bar.update(len(chunk))  # new
                             f.write(chunk)
                 progress_bar.close()  # new
-                if total_size_in_bytes != 0 and progress_bar.n != total_size_in_bytes:
+                if (
+                    total_size_in_bytes != 0
+                    and progress_bar.n != total_size_in_bytes
+                ):
                     print(
-                        "Warning: Downloaded size {progress_bar.n} does not match {total_size_in_bytes}."
+                        "Warning: Downloaded size {progress_bar.n} "
+                        "does not match {total_size_in_bytes}."
                     )
 
             elif "Manual" in url:
                 raise urllib.error.URLError(url)
             else:
                 print("Downloading " + url + " to " + fpath)
-                urllib.request.urlretrieve(url, fpath, reporthook=gen_bar_updater())
+                urllib.request.urlretrieve(
+                    url, fpath, reporthook=gen_bar_updater()
+                )
         except (urllib.error.URLError, IOError) as e:
             if url[:5] == "https":
                 url = url.replace("https:", "http:")
@@ -85,7 +92,9 @@ def download_url(url, root, filename=None, md5=None, total_size=None):
                     "Failed download. Trying https -> http instead."
                     " Downloading " + url + " to " + fpath
                 )
-                urllib.request.urlretrieve(url, fpath, reporthook=gen_bar_updater())
+                urllib.request.urlretrieve(
+                    url, fpath, reporthook=gen_bar_updater()
+                )
             else:
                 raise e
         # check integrity of downloaded file
@@ -144,13 +153,14 @@ class NeuromorphicDataset(data.Dataset):
                     self._create_hdf5()  #
                 else:
                     raise Exception(
-                        "File {} does not exist and download_and_create is False".format(
-                            root
-                        )
+                        "File {} does not exist and download_and_create "
+                        "is False".format(root)
                     )
 
         has_transforms = transforms is not None
-        has_separate_transform = transform is not None or target_transform is not None
+        has_separate_transform = (
+            transform is not None or target_transform is not None
+        )
         if has_transforms and has_separate_transform:
             raise ValueError(
                 "Only transforms or transform/target_transform can "
@@ -212,7 +222,9 @@ class NeuromorphicDataset(data.Dataset):
         if res is False:
             for _, _, filename in self._resources_url:
                 extract_root = self.directory
-                archive = os.path.join(extract_root, filename).replace("\\", "/")
+                archive = os.path.join(extract_root, filename).replace(
+                    "\\", "/"
+                )
                 print("Extracting {} to {}...".format(archive, extract_root))
                 _extract_archive(archive, extract_root, remove_finished=False)
         return res
@@ -225,7 +237,10 @@ class NeuromorphicDataset(data.Dataset):
             os.makedirs(self.directory, exist_ok=True)
             for url, md5, filename in self._resources_url:
                 download_and_extract_archive(
-                    url, download_root=self.directory, filename=filename, md5=md5
+                    url,
+                    download_root=self.directory,
+                    filename=filename,
+                    md5=md5,
                 )
             return False
 
@@ -255,13 +270,76 @@ def _extract_archive(from_path, to_path=None, remove_finished=False):
 
     if _is_tar(from_path):
         with tarfile.open(from_path, "r") as tar:
-            tar.extractall(path=to_path)
+
+            def is_within_directory(directory, target):
+
+                abs_directory = os.path.abspath(directory)
+                abs_target = os.path.abspath(target)
+
+                prefix = os.path.commonprefix([abs_directory, abs_target])
+
+                return prefix == abs_directory
+
+            def safe_extract(
+                tar, path=".", members=None, *, numeric_owner=False
+            ):
+
+                for member in tar.getmembers():
+                    member_path = os.path.join(path, member.name)
+                    if not is_within_directory(path, member_path):
+                        raise Exception("Attempted Path Traversal in Tar File")
+
+                tar.extractall(path, members, numeric_owner=numeric_owner)
+
+            safe_extract(tar, path=to_path)
     elif _is_targz(from_path) or _is_tgz(from_path):
         with tarfile.open(from_path, "r:gz") as tar:
-            tar.extractall(path=to_path)
+
+            def is_within_directory(directory, target):
+
+                abs_directory = os.path.abspath(directory)
+                abs_target = os.path.abspath(target)
+
+                prefix = os.path.commonprefix([abs_directory, abs_target])
+
+                return prefix == abs_directory
+
+            def safe_extract(
+                tar, path=".", members=None, *, numeric_owner=False
+            ):
+
+                for member in tar.getmembers():
+                    member_path = os.path.join(path, member.name)
+                    if not is_within_directory(path, member_path):
+                        raise Exception("Attempted Path Traversal in Tar File")
+
+                tar.extractall(path, members, numeric_owner=numeric_owner)
+
+            safe_extract(tar, path=to_path)
     elif _is_tarxz(from_path):
         with tarfile.open(from_path, "r:xz") as tar:
-            tar.extractall(path=to_path)
+
+            def is_within_directory(directory, target):
+
+                abs_directory = os.path.abspath(directory)
+                abs_target = os.path.abspath(target)
+
+                prefix = os.path.commonprefix([abs_directory, abs_target])
+
+                return prefix == abs_directory
+
+            def safe_extract(
+                tar, path=".", members=None, *, numeric_owner=False
+            ):
+
+                for member in tar.getmembers():
+                    member_path = os.path.join(path, member.name)
+                    if not is_within_directory(path, member_path):
+                        raise Exception("Attempted Path Traversal in Tar File")
+
+                tar.extractall(path, members, numeric_owner=numeric_owner)
+
+            safe_extract(tar, path=to_path)
     elif _is_gzip(from_path):
         to_path = os.path.join(
             to_path, os.path.splitext(os.path.basename(from_path))[0]
@@ -337,3 +415,35 @@ class DownloadProgressBar(tqdm):
         if tsize is not None:
             self.total = tsize
         self.update(b * bsize - self.n)
+
+
+def check_integrity(fpath, md5=None) -> bool:
+    if not os.path.isfile(fpath):
+        return False
+    if md5 is None:
+        return True
+    return check_md5(fpath, md5)
+
+
+def calculate_md5(fpath, chunk_size=1024 * 1024):
+    md5 = hashlib.md5()
+    with open(fpath, "rb") as f:
+        for chunk in iter(lambda: f.read(chunk_size), b""):
+            md5.update(chunk)
+    return md5.hexdigest()
+
+
+def check_md5(fpath, md5, **kwargs):
+    return md5 == calculate_md5(fpath, **kwargs)
+
+
+def gen_bar_updater():
+    pbar = tqdm(total=None)
+
+    def bar_update(count, block_size, total_size):
+        if pbar.total is None and total_size:
+            pbar.total = total_size
+        progress_bytes = count * block_size
+        pbar.update(progress_bytes - pbar.n)
+
+    return bar_update
